#ifndef __BASICS_H__
#define __BASICS_H__

struct point {
  int x ;
  int y ;
};

struct ball {
  struct point position ;   /* En coordonnes pixels. */
  struct point direction ;
  struct point initial_direction ;
  struct sSpriteEntry *oam_copy_data ;
};

enum work_flag { Work_done, Work_todo };

struct player {
  enum work_flag input_handled ;
  struct point start_touch ;
  struct point end_touch ;      // Significatif si input_handled != Work_done.
};

enum line_orientation {
  Line_none,
  Line_h,
  Line_v
};

typedef u8 extremities ;
#define NONE_MASK 0x00
#define POS_MASK 0x02
#define NEG_MASK 0x01
#define BOTH_MASK (POS_MASK | NEG_MASK)
#define CLOSE_POS(e) (e) |= POS_MASK
#define CLOSE_NEG(e) (e) |= NEG_MASK
#define OPEN_BOTH(e) (e) = 0x00

struct liner {
  enum line_orientation orient ;
  extremities extrems ;           // Significatif si orient != Line_none.
  struct point origin ;           // Significatif si orient != Line_none.
  int current_moving_pos_coord ;  // Significatif si orient != Line_none.
  int current_moving_neg_coord ;  // Significatif si orient != Line_none.
};

#define BALL_WIDTH 8
#define BALL_HEIGHT 8
#define MAX_NB_BALLS 6
#define MAX_ABS_SPEED 3
#define LINE_SPEED 1

#define BOARD_WIDTH (256/8)
#define BOARD_HEIGHT (192/8) 

enum goodies_kind {
  /* Dure contrainte par le temps (3 lignes russies). */
  Goodies_flip_screens = 0,
  Goodies_perfor_balls = 1,
  Goodies_dizzy_balls = 2,
  Goodies_speed_balls = 3,
  Goodies_flip_xy = 4,
  Goodies_slow_lines = 5,
  /* Dure non contrainte par le temps. */
  Goodies_random_explode = 6,
};
#define NB_GOODIES 7
#define NB_TIME_CONSTRAINTED_GOODIES 6

enum game_type {
  Game_standard,
  Game_arcade
};

#define GOODIES_MASK_TO_STATUS_MASK(g) (1 << g)
/* Contrainte de temps: 3 lignes compltes. */
#define GOODIES_DURATION 2

struct status {
  /* Bits mis si le goodies est actuellement actif. */
  u16 status ;
  /* Nombre de lignes devant tre faites pour la fin de l'effet fin. */
  u16 ends[NB_TIME_CONSTRAINTED_GOODIES] ;
} ;

/* Nombre de secondes pour finir le niveau avant sanction */
/* (*2 car le timer compte en demi-secondes).             */
#define KILL_DELAY (15 * 2)

#define TO_FILL_PERCENT 80

struct game {
  struct player player ;
  struct liner liner ;
  struct status status ;
  enum game_type game_type ;
  u32 kill_time ;
  u8 lives ;
  u8 nb_balls ;
  u16 level ;
  u32 prev_score ;
  u32 score ;
  u16 nb_filled_cells ;
  u16 nb_lines_done ;
  u8 board[BOARD_WIDTH][BOARD_HEIGHT] ;
  struct ball balls[MAX_NB_BALLS] ;
};

enum end_code {
  Game_on,
  Game_lost,
  Game_won,
  Game_restart
};

/* Reprsente l'absence de mur. */
#define CELL_FREE 0
/* Reprsente un bloc de mur en construction (pas encore solidifi). */
#define CELL_WALL 1
/* Reprsente un bloc de mur construit (solidifi). */
#define CELL_BORDER 2
/* Sert  signifi, lors de la recherche de collision qu'il y a une balle. */
#define CELL_BALL 3
/* CELL_LIMIT est comme CELL_BORDER sauf que a dsigne le bord  */
/* de l'cran et donc ne peut pas tre attaqu par les vers, les */
/* explosions ou les balles perforantes.                         */
#define CELL_LIMIT 4

extern int max (int, int) ;
extern int min (int, int) ;

#endif
