#include <nds.h>
#include <stdio.h>
#include "basics.h"
#include "walls_gfx.h"


void add_wall_block (u8 board[BOARD_WIDTH][BOARD_HEIGHT], u16 x, u16 y)
{
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;

  board[x][y] = CELL_WALL ;
  map[(y * (256 / 8)) + x] =
    BUILDING_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
}

void rem_wall_block (u8 board[BOARD_WIDTH][BOARD_HEIGHT], u16 x, u16 y)
{
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;

  board[x][y] = CELL_FREE ;
  map[(y * (256 / 8)) + x] =
    EMPTY_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
}


void vfill_border (struct game *game, u16 x, u16 ymin, u16 ymax)
{
  u16 y ;
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;
  u16 *ptr ;
  u8 filled ;

  ptr = &(map[(ymin * (256 / 8)) + x]) ;
  for (y = ymin; y <= ymax; y++) {
    game->board[x][y] = CELL_BORDER ;
    *ptr = BUILT_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
    ptr += (256 / 8) ;
  }

  filled = (ymax - ymin) + 1 ;
  game->nb_filled_cells += filled ;
  game->score += filled ;
}



void hfill_border (struct game *game, u16 xmin, u16 xmax, u16 y)
{
  u16 x ;
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;
  u16 *ptr ;
  u8 filled ;

  ptr = &(map[(y * (256 / 8)) + xmin]) ;
  for (x = xmin; x <= xmax; x++) {
    game->board[x][y] = CELL_BORDER ;
    *ptr = BUILT_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
    ptr++ ;
  }

  filled = (xmax - xmin) + 1 ;
  game->nb_filled_cells += filled ;
  game->score += filled ;
}




static void fill (u8 board[BOARD_WIDTH][BOARD_HEIGHT], u16 x, u16 y)
{
  int _x ;
  int stopx ;

  if (board[x][y] != CELL_FREE) return ;

  /* Dplacement vers la gauche. */
  for (_x = x ; board[_x][y] == CELL_FREE; _x--)
    board[_x][y] = CELL_BORDER ;
  stopx = _x ;

  /* Remplissage au dessus. */
  for (_x = x; _x > stopx; _x--)
    if (board[_x][y-1] == CELL_FREE) fill (board, _x, y-1) ;
  
  /* Remplissage en dessous. */
  for (_x = x; _x > stopx; _x--)
    if (board[_x][y+1] == CELL_FREE) fill (board, _x, y+1) ;
  
  /* Dplacement vers la droite. */
  for (_x = x + 1; board[_x][y] == CELL_FREE; _x++)
    board[_x][y] = CELL_BORDER ;
  stopx = _x ;
   
  /* Remplissage au dessus. */
  for (_x = x+1; _x < stopx; _x++)
    if (board[_x][y-1]== CELL_FREE) fill (board, _x, y-1) ;
  
  /* Remplissage en dessous. */
  for (_x = x+1; _x < stopx; _x++)
    if (board[_x][y+1] == CELL_FREE) fill (board, _x, y+1) ;
}


/* Tableau temporaire de recopie de l'espace de jeu rel. */
static u8 tmp_board[BOARD_WIDTH][BOARD_HEIGHT] ;


void fill_board (struct game *game)
{
  u16 x, y, i ;
  u16 *tile_addr ;
  u16 nb_filled_cells = 0 ;
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;
  u8 nb_balls = game->nb_balls ;

  /* Copie du jeu en buffer. */
  for (y = 0; y < BOARD_HEIGHT; y++) {
    for (x = 0; x < BOARD_WIDTH; x++)
      tmp_board[x][y] = game->board[x][y] ;
  }

  /* On remplit toutes les zones o il y a une balle (en temporaire). */
  for (i = 0; i < nb_balls; i++)
    fill (tmp_board, (game->balls[i].position.x / BALL_WIDTH),
	  (game->balls[i].position.y / BALL_HEIGHT)) ;

  /* Maintenant le reste dans le vrai jeu devient rempli. */
  for (y = 0; y < BOARD_HEIGHT; y++) {
    tile_addr = &(map[y * (256 / 8)]) ;
    for (x = 0; x < BOARD_WIDTH; x++) {
      if (tmp_board[x][y] == CELL_FREE) {
	game->board[x][y] = CELL_BORDER ;
	*tile_addr = BUILT_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
	nb_filled_cells++ ;
      }
      tile_addr++ ;
    }
  }

  /* Mise  jour du nombre de blocks remplis une bonne fois pour toute. */
  game->nb_filled_cells += nb_filled_cells ;
  game->score += nb_filled_cells ;
}

