#include <nds.h>
#include <nds/arm9/sound.h>
#include <stdlib.h>
#include <string.h>
#include "basics.h"
#include "global.h"
#include "walls_gfx.h"
#include "sprite_red_ball_gfx.h"
#include "sprite_maelstrom_gfx.h"
#include "sprite_swap_screen_gfx.h"
#include "sprite_invert_pen_gfx.h"
#include "sprite_dizzy_ball_gfx.h"
#include "sprite_earth_quake_gfx.h"
#include "sprite_slow_lines_gfx.h"
#include "sprite_speed_balls_gfx.h"
#include "sprite_radioac_balls_gfx.h"
#include "laugh.h"
#include "gfx_load.h"
#include "goodies.h"
#include "wav_earthquake.h"
#include "wav_slow_lines.h"
#include "wav_invert_pen.h"
#include "wav_swap_screens.h"
#include "wav_speed_balls.h"
#include "wav_perfo_balls.h"



short global_line_speed_delay = 0 ;




/* Explosion alatoire de blocs. */
static void random_explode (struct game *game)
{
  u8 turn, x, y, i, j, width, height ;
  u16 old_blending_mode_sub ;
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;

  old_blending_mode_sub = SUB_BLEND_CR ;
  SUB_BLEND_Y = 15 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;

  /* Chargement des donnes sprites de tourbillon en mmoire graphique. */
  load_sprites_16
    (SPRITES_SLIDING_LOADING_OFFSET,
     SPRITE_MAELSTROM_NB_TILES,
     sprite_maelstrom_tiles,
     SPRITES_SLIDING_PALETTE_SLOT,
     SPRITE_MAELSTROM_PALETTE_SIZE,
     sprite_maelstrom_palette,
     Video_sub) ;

  /* Activation du sprite de tourbillon. */
  global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[0] =
    ATTR0_COLOR_16 | ATTR0_SQUARE ;                  /* Y  0 au dbut. */
  global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[1] =
    ATTR1_SIZE_64 ;                                  /* X  0 au dbut. */
  global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[2] =
    SPRITES_SLIDING_TILE_INDEX |
    (SPRITES_SLIDING_PALETTE_SLOT << 12) ;
  global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[3] = 0 ;
  for (x = SPRITES_SLIDING_OAM_ENTRY_INDEX + 1 ; x < 128; x++)
    global_sprites_copy[x].attribute[0] = ATTR0_DISABLED ;
  /* On ne change pas les autres sprites. */

  /* Au max, 15 occurrences de blocs peuvent tre dtruits. */
  for (turn = 0; turn < 15; turn++) {
    /* Position top-left de dpart de l'aire dtruite. */
    x = 1 + ((u8) (rand () % (BOARD_WIDTH - 2))) ;
    y = 1 + ((u8) (rand () % (BOARD_HEIGHT - 2))) ;
    /* Taille de l'aire dtruite. */
    width = 1 + ((u8) (rand () % 10)) ;
    height = 1 + ((u8) (rand () % 10)) ;
    for (j = y; j < y + height; j++) {
      for (i = x ; i < x + width; i++) {
	/* Test pour ne pas sortir de la zone de jeu de l'cran. */
	if ((i < BOARD_WIDTH) && (j < BOARD_HEIGHT)) {
	  /* On ne dtruit que des murs solidifis. */
	  if (game->board[i][j] == CELL_BORDER) {
	    /* On ne rduit quand mme pas le score. */
	    game->nb_filled_cells-- ;
	    game->board[i][j] = CELL_FREE ;
	    map[(j * (256 / 8)) + i] =
	      EMPTY_WALL_TILE_INDEX | (WALLS_PALETTE_SLOT << 12) ;
	  }  /* Fin de if (game->board[i][j] == CELL_BORDER). */
	}    /* Fin de if ((i < BOARD_WIDTH) && (j < BOARD_HEIGHT)). */
      }      /* Fin de for (i = x ; i < x + width; i++). */

      /* Tremblement de l'cran  chaque ligne traite. */
      SUB_BLEND_Y = (u16) ((rand ()) % 16) ;
      SUB_BG0_X0 = SUB_BG1_X0 = ((s16) ((rand ()) % 8)) - 4 ;
      SUB_BG0_Y0 = SUB_BG1_Y0 = ((s16) ((rand ()) % 8)) - 4 ;
    }   /* Fin de for (j = y; j < y + height; j++). */

    /* Sprite de tourbillon. */
    global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[1] =
      (global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[1] &
       0xFF00) | ((x * 8) & 0x00FF) ;
    global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[0] =
      (global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[0] &
       0xFE00) | ((y * 8) & 0x01FF) ;
    swiWaitForVBlank () ;
    memcpy
      ((void*) OAM_SUB, (void*) global_sprites_copy,
       128 * sizeof (struct sSpriteEntry)) ;
  }          /* Fin de for (turn = 0; turn < 15; turn++). */

  /* Dsactivation du sprite en fin. */
  global_sprites_copy[SPRITES_SLIDING_OAM_ENTRY_INDEX].attribute[0] =
    ATTR0_DISABLED ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;

  /* Rtablissement des dcalages et du blending mode d'origines. */
  SUB_BG0_X0 = SUB_BG1_X0 = 0 ;
  SUB_BG0_Y0 = SUB_BG1_Y0 = 0 ;
  SUB_BLEND_CR = old_blending_mode_sub ;
}




void launch_malus (struct game *game)
{
  enum goodies_kind malus ;
  short i ;

  /* Apparemment, rand est bugge et donne toujours un multiple de 8. */
  malus = (enum goodies_kind) (((rand ()) + DISP_Y) % NB_GOODIES) ;

  switch (malus) {
  case Goodies_flip_screens:
    playGenericSound
      (g_wav_swap_screens, WAV_SWAP_SCREENS_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_swap_screen_palette, SPRITE_SWAP_SCREEN_PALETTE_SIZE,
       sprite_swap_screen_tiles, SPRITE_SWAP_SCREEN_NB_TILES) ;
    /* On n'inverse que si on n'est pas dj en mode invers. */
    if (!(game->status.status &
	  (GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_screens))))
      lcdSwap () ;
    game->status.ends[Goodies_flip_screens] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_screens) ;
    break ;

  case Goodies_perfor_balls:
    playGenericSound
      (g_wav_perfo_balls, WAV_PERFO_BALLS_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_radioac_balls_palette, SPRITE_RADIOAC_BALLS_PALETTE_SIZE,
       sprite_radioac_balls_tiles, SPRITE_RADIOAC_BALLS_NB_TILES) ;
    game->status.ends[Goodies_perfor_balls] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_perfor_balls) ;
    break ;

  case Goodies_dizzy_balls:
    slide_sprite
      (sprite_dizzy_ball_palette, SPRITE_DIZZY_BALL_PALETTE_SIZE,
       sprite_dizzy_ball_tiles, SPRITE_DIZZY_BALL_NB_TILES) ;
    game->status.ends[Goodies_dizzy_balls] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_dizzy_balls) ;
    break ;

  case Goodies_speed_balls:
    playGenericSound
      (g_wav_speed_balls, WAV_SPEED_BALLS_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_speed_balls_palette, SPRITE_SPEED_BALLS_PALETTE_SIZE,
       sprite_speed_balls_tiles, SPRITE_SPEED_BALLS_NB_TILES) ;
    for (i = 0; i < game->nb_balls; i++) {
      if ((game->balls[i].direction.x < 0) &&
	  (game->balls[i].direction.x > (-(MAX_ABS_SPEED))))
	game->balls[i].direction.x-- ;
      else {
	if ((game->balls[i].direction.x >= 0) &&
	    (game->balls[i].direction.x < (MAX_ABS_SPEED)))
	  game->balls[i].direction.x++ ;
      }
      if ((game->balls[i].direction.y < 0) &&
	  (game->balls[i].direction.y > (-(MAX_ABS_SPEED))))
	game->balls[i].direction.y-- ;
      else {
	if ((game->balls[i].direction.y >= 0) &&
	    (game->balls[i].direction.y < (MAX_ABS_SPEED)))
	  game->balls[i].direction.y++ ;
      }
    }
    game->status.ends[Goodies_speed_balls] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_speed_balls) ;
    break ;

  case Goodies_flip_xy:
    playGenericSound (g_wav_invert_pen, WAV_INVERT_PEN_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_invert_pen_palette, SPRITE_INVERT_PEN_PALETTE_SIZE,
       sprite_invert_pen_tiles, SPRITE_INVERT_PEN_NB_TILES) ;
    game->status.ends[Goodies_flip_xy] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_xy) ;
    break ;

  case Goodies_slow_lines:
    playGenericSound (g_wav_slow_lines, WAV_SLOW_LINES_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_slow_lines_palette, SPRITE_SLOW_LINES_PALETTE_SIZE,
       sprite_slow_lines_tiles, SPRITE_SLOW_LINES_NB_TILES) ;
    if (global_line_speed_delay < MAX_LINE_SPEED_DELAY)
      global_line_speed_delay += 4 ;
    game->status.ends[Goodies_slow_lines] =
      game->nb_lines_done + GOODIES_DURATION ;
    game->status.status |= GOODIES_MASK_TO_STATUS_MASK (Goodies_slow_lines) ;
    break ;

  case Goodies_random_explode:
    playGenericSound (g_wav_earthquake, WAV_EARTHQUAKE_LENGTH * sizeof (u32)) ;
    slide_sprite
      (sprite_earth_quake_palette, SPRITE_EARTH_QUAKE_PALETTE_SIZE,
       sprite_earth_quake_tiles, SPRITE_EARTH_QUAKE_NB_TILES) ;
    random_explode (game) ;
    break ;
  }
}

