#include <nds.h>
#include <stdlib.h>
#include <string.h>
#include "basics.h"
#include "global.h"
#include "gfx_load.h"
#include "game_bottom_gfx.h"
#include "game_top_gfx.h"
#include "large_fonts_gfx.h"
#include "sprite_laughing_ball_gfx.h"
#include "sprite_maelstrom_gfx.h"
#include "sprite_swap_screen_gfx.h"
#include "sprite_invert_pen_gfx.h"
#include "sprite_dizzy_ball_gfx.h"
#include "sprite_earth_quake_gfx.h"
#include "sprite_red_ball_gfx.h"
#include "walls_gfx.h"
#include "printing.h"
#include "time.h"
#include "goodies.h"



static void allocate_balls (u8 nb_balls, struct ball *balls)
{
  u16 i ;
  u8 rnd ;

  for (i = 0; i < nb_balls; i++) {
    balls[i].oam_copy_data = &global_sprites_copy[i] ;

    balls[i].position.x = // -2 au lieu de -1 car les balles font 8 de diamtre
      max ((1 * 8), ((rand () + DISP_Y) % ((BOARD_WIDTH - 2) * 8))) ;
    balls[i].position.y = // -2 au lieu de -1 car les balles font 8 de diamtre
      max ((1 * 8), ((rand () + DISP_Y) % ((BOARD_HEIGHT - 2) * 8))) ;
    balls[i].direction.x =
      ((rand () + DISP_Y) % (2 * MAX_ABS_SPEED)) - MAX_ABS_SPEED ;
    balls[i].direction.y =
      ((rand () + DISP_Y) % (2 * MAX_ABS_SPEED)) - MAX_ABS_SPEED ;
    /* Pour viter d'avoir des balles de direction nulle. */
    if (balls[i].direction.x == 0) {
      rnd = (u8) (rand () % 2) ;
      switch (rnd) {
      case 0: balls[i].direction.x++ ; break ;
      default: balls[i].direction.x-- ; break ;
      }
    }

    if (balls[i].direction.y == 0) {
      rnd = (u8) (rand () % 2) ;
      switch (rnd) {
      case 0: balls[i].direction.y++ ; break ;
      default: balls[i].direction.y-- ; break ;
      }
    }

    /* On mmorise les vitesses initiales. */
    balls[i].initial_direction.x = balls[i].direction.x ;
    balls[i].initial_direction.y = balls[i].direction.y ;

    global_sprites_copy[i].attribute[0] =
      ATTR0_COLOR_16 | ATTR0_SQUARE | (balls[i].position.y & 0x00FF) ;
    global_sprites_copy[i].attribute[1] =
      ATTR1_SIZE_8 | (balls[i].position.x & 0x01FF) ;
    global_sprites_copy[i].attribute[2] =
      SPRITE_RED_BALL_TILE_INDEX | (SPRITE_RED_BALL_PALETTE_SLOT << 12) ;
    global_sprites_copy[i].attribute[3] = 0 ;
  }

  for (; i < 128; i++) global_sprites_copy[i].attribute[0] = ATTR0_DISABLED ;
  swiWaitForVBlank () ;
  memcpy
    ((void*) OAM_SUB, (void*) global_sprites_copy,
     128 * sizeof (struct sSpriteEntry)) ;
}





static void init_board (u8 board[BOARD_WIDTH][BOARD_HEIGHT])
{
  int x, y ;
  u16 *map = (u16*) SCREEN_BASE_BLOCK_SUB (1) ;

  for (y = 0; y < BOARD_HEIGHT; y++) {
    for (x = 0; x < BOARD_WIDTH; x++) {
      board[x][y] = CELL_FREE ;
      map[(y * (256 / 8)) + x] = EMPTY_WALL_TILE_INDEX ;
    }
  }
  
  for (x = 0; x < BOARD_WIDTH; x++) {
    board[x][0] = CELL_LIMIT ;
    board[x][(BOARD_HEIGHT-1)] = CELL_LIMIT ;
  }

  for (y = 1; y < (BOARD_HEIGHT -1) ; y++) {
    board[0][y] = CELL_LIMIT ;
    board[(BOARD_WIDTH - 1)][y] = CELL_LIMIT ;
  }
}





void start_newgame (enum end_code game_end_code, enum game_type game_kind)
{
  switch (game_end_code) {
  case Game_won:
    /* On gagne 1 vie tous les 3 niveaux. */
    global_game.level++ ;
    global_game.prev_score = global_game.score ;
    if ((global_game.level % 3) == 0) global_game.lives++ ;
    break ;

  case Game_lost:
    /* Dbut from scratch. */
    global_game.lives = 3 ;
    global_game.score = 0 ;
    global_game.prev_score = 0 ;
    global_game.level = 1 ;
    break ;

  case Game_restart:
    /* On recommence le niveau car pnalit. */
    global_game.score = global_game.prev_score ;
    break ;

  case Game_on:
    /* Ne devrait jamais se produire. */
    break ;
  }

  global_game.kill_time = elapsed_semi_seconds + KILL_DELAY ;
  global_game.nb_balls = 1 + (global_game.level / 8) ;
  /* Si le niveau nous impose plus de balles que le max, */
  /* alors on tire  le nombre de balles au hasard.       */
  if (global_game.nb_balls > MAX_NB_BALLS)
    global_game.nb_balls = (u8) (1 + (rand () % (MAX_NB_BALLS - 1))) ;

  allocate_balls (global_game.nb_balls, global_game.balls) ;
  init_board (global_game.board) ;
  global_game.liner.orient = Line_none ;
  global_game.player.input_handled = Work_done ;
  global_game.nb_filled_cells = (BOARD_WIDTH + BOARD_HEIGHT - 2) * 2 ;
  global_game.nb_lines_done = 0 ;
  /* Rinitialisation de tous les effets. */
  global_game.status.status = 0x0000 ;
  global_game.game_type = game_kind ;
  /* Rinitialisation de la tempo de ralentissement des balles. */
  global_line_speed_delay = 0 ;

  /* Mise  jour des infos numriques  l'cran. */
  large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		   ((global_game.kill_time - elapsed_semi_seconds) / 2),
		   NUMERICS_X_OFFSET, TIMER_Y_OFFSET) ;
  large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		   global_game.score, NUMERICS_X_OFFSET, SCORE_Y_OFFSET) ;
  large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		   global_game.lives, NUMERICS_X_OFFSET, LIVES_Y_OFFSET) ;
  large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		   global_game.level, NUMERICS_X_OFFSET, LEVEL_Y_OFFSET) ;
}




void init_playground_score_area ()
{
  u16 i ;

  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK (1),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK (0),   // MAP (2ko aligne) (adresse)
     GAME_TOP_PALETTE_SIZE,
     game_top_palette,
     NB_GAME_TOP_TILES,
     game_top_tiles,
     GAME_TOP_MAP_WIDTH,
     GAME_TOP_MAP_HEIGHT,
     game_top_map,
     Video_main) ;

  load_palette_tile_map_16
    ((u16*) CHAR_BASE_BLOCK (2),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK (1),   // MAP (2ko aligne) (adresse)
     LARGE_FONTS_PALETTE_SLOT,
     LARGE_FONTS_PALETTE_SIZE,
     large_fonts_palette,
     NB_LARGE_FONTS_TILES,
     large_fonts_tiles,
     0,                                  // On construira la map  la pogne.
     0,
     NULL,
     Video_main) ;

  /* On efface la map destine aux fonts de score etc  la main. */
  for (i = 0; i < (32 * 24); i++) {
    ((u16*) SCREEN_BASE_BLOCK (1))[i] =
      (u16) (EMPTY_FONT_TILE_INDEX | (LARGE_FONTS_PALETTE_SLOT << 12)) ;
  }

  BG0_CR =
      BG_MAP_BASE (0)        // Map (pas adresse).
    | BG_TILE_BASE (1)       // Tiles (pas adresse).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  BG1_CR =
      BG_MAP_BASE (1)        // Map (pas adresse).
    | BG_TILE_BASE (2)       // Tiles (pas adresse).
    | BG_PRIORITY_2          // Priorit moins faible.
    | BG_COLOR_16            // 16 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels
}



void init_playground ()
{
  short i ;
  u16 old_blending_mode, old_blending_mode_sub ;

  /* cran infrieur. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK_SUB (1),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK_SUB (0),   // MAP (2ko aligne) (adresse)
     GAME_BOTTOM_PALETTE_SIZE,
     game_bottom_palette,
     NB_GAME_BOTTOM_TILES,
     game_bottom_tiles,
     GAME_BOTTOM_MAP_WIDTH,
     GAME_BOTTOM_MAP_HEIGHT,
     game_bottom_map,
     Video_sub) ;

  load_palette_tile_map_16
    ((u16*) CHAR_BASE_BLOCK_SUB (2),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK_SUB (1),   // MAP (2ko aligne) (adresse)
     WALLS_PALETTE_SLOT,
     WALLS_PALETTE_SIZE,
     walls_palette,
     NB_WALLS_TILES,
     walls_tiles,
     0,                                  // On construira la map  la pogne.
     0,
     NULL,
     Video_sub) ;

  SUB_BG0_CR =
      BG_MAP_BASE (0)        // Map (pas adresse).
    | BG_TILE_BASE (1)       // Tiles (pas adresse).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  SUB_BG1_CR =
      BG_MAP_BASE (1)        // Map (pas adresse).
    | BG_TILE_BASE (2)       // Tiles (pas adresse).
    | BG_PRIORITY_2          // Priorit moins faible.
    | BG_COLOR_16            // 16 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  for (i = 0; i < (32 * 24); i++) {
    /* On efface la map du layer de dessin des blocs  la main. */
    ((u16*) SCREEN_BASE_BLOCK_SUB (1))[i] =
      (u16) (EMPTY_FONT_TILE_INDEX | (LARGE_FONTS_PALETTE_SLOT << 12)) ;
  }

  /* cran suprieur. */
  init_playground_score_area () ;

  /* Sprite des balles normales. */
  load_sprites_16
    (SPRITE_RED_BALL_LOADING_OFFSET,
     SPRITE_RED_BALL_NB_TILES,
     sprite_red_ball_tiles,
     SPRITE_RED_BALL_PALETTE_SLOT,
     SPRITE_RED_BALL_PALETTE_SIZE,
     sprite_red_ball_palette,
     Video_sub) ;

  /* Apparition de l'cran en fading. */
  old_blending_mode = BLEND_CR ;
  old_blending_mode_sub = SUB_BLEND_CR ;
  BLEND_Y = 15 ;
  SUB_BLEND_Y = 15 ;
  BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;

  videoSetMode (MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE) ;
  videoSetModeSub (MODE_0_2D | DISPLAY_SPR_1D_LAYOUT | DISPLAY_SPR_ACTIVE |
		   DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE) ;

  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }

  BLEND_CR = old_blending_mode ;
  SUB_BLEND_CR = old_blending_mode_sub ;
}
