#include <nds.h>
#include <nds/arm9/sound.h>
#include <string.h>
#include "basics.h"
#include "global.h"
#include "gfx_load.h"
#include "intro_bottom_gfx.h"
#include "intro_top_gfx.h"
#include "scores.h"
#include "wav_screen_touch.h"



#define PEN_HIT (pen_curr & ~pen_prev)

static bool wait_intro_input (enum game_type *gtype_out)
{
  u16 x, y ;
  touchPosition stylus_xy ;
  u16 pen_prev = false ;
  u16 pen_curr = false ;

  keysInit () ;
  for (;;) {
    scanKeys() ;

    /* L'appui sur START fair continuer par dfaut. */
    if (keysDown () & (KEY_START)) return (true) ;

    stylus_xy = touchReadXY () ;
    pen_prev = pen_curr ;
    pen_curr = (keysDown () & KEY_TOUCH) ;

    if (PEN_HIT) {
      x = stylus_xy.px ;
      y = stylus_xy.py ;

      if ((x >= (BUTTONS_X * 8)) && (x <= ((BUTTONS_X + BUTTONS_WIDTH) * 8))) {
	/* On est positionn sur une abscisse de bouton. */
	if ((y >= BUTTON_STANDARD_Y * 8) &&
	    (y < ((BUTTON_STANDARD_Y + BUTTONS_HEIGHT) * 8))) {
	  playGenericSound
	    (g_wav_screen_touch, WAV_SCREEN_TOUCH_LENGTH * sizeof (u32)) ;
	  *gtype_out = Game_standard ;
	  return (true) ;                    /* Mode standard. */
	}

	if ((y >= BUTTON_ARCADE_Y * 8) &&
	    (y < ((BUTTON_ARCADE_Y + BUTTONS_HEIGHT) * 8))) {
	  playGenericSound
	    (g_wav_screen_touch, WAV_SCREEN_TOUCH_LENGTH * sizeof (u32)) ;
	  *gtype_out = Game_arcade ;
	  return (true) ;                      /* Mode arcade. */
	}

	if ((y >= BUTTON_SCORES_Y * 8) &&
	    (y < ((BUTTON_SCORES_Y + BUTTONS_HEIGHT) * 8))) {
	  playGenericSound
	    (g_wav_screen_touch, WAV_SCREEN_TOUCH_LENGTH * sizeof (u32)) ;
	  show_scores () ;                    /* Afficher les highscores. */
	  return (false) ;
	} /* Fin de if ((y >= BUTTON_SCORES_Y * 8) && ... */
      }   /* Fin de if ((x >= (BUTTONS_X * 8)) && (x <= ((BUTTOS_X + BUT ... */
    }     /* Fin de if (keysHeld () & KEY_TOUCH). */
  }
}



enum game_type intro ()
{
  u16 old_blending_mode, old_blending_mode_sub ;
  enum game_type selected_game_type = Game_standard ;
  short i ;

 dirty_restart_intro:
  /* cran infrieur. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK_SUB (1),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK_SUB (0),   // MAP (2ko aligne) (adresse)
     INTRO_BOTTOM_PALETTE_SIZE,
     intro_bottom_palette,
     NB_INTRO_BOTTOM_TILES,
     intro_bottom_tiles,
     INTRO_BOTTOM_MAP_WIDTH,
     INTRO_BOTTOM_MAP_HEIGHT,
     intro_bottom_map,
     Video_sub) ;

  SUB_BG0_CR =
      BG_MAP_BASE (0)        // Map (pas adresse).
    | BG_TILE_BASE (1)       // Tiles (pas adresse).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  /* cran suprieur. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK (1),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK (0),   // MAP (2ko aligne) (adresse)
     INTRO_TOP_PALETTE_SIZE,
     intro_top_palette,
     NB_INTRO_TOP_TILES,
     intro_top_tiles,
     INTRO_TOP_MAP_WIDTH,
     INTRO_TOP_MAP_HEIGHT,
     intro_top_map,
     Video_main) ;

  BG0_CR =
      BG_MAP_BASE (0)        // Map (pas adresse).
    | BG_TILE_BASE (1)       // Tiles (pas adresse).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels


  /* Apparition de l'cran en fading. */
  old_blending_mode = BLEND_CR ;
  old_blending_mode_sub = SUB_BLEND_CR ;
  BLEND_Y = 15 ;
  SUB_BLEND_Y = 15 ;
  BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;

  videoSetMode (MODE_0_2D | DISPLAY_BG0_ACTIVE) ;
  videoSetModeSub (MODE_0_2D | DISPLAY_BG0_ACTIVE) ;

  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }

  BLEND_CR = old_blending_mode ;
  SUB_BLEND_CR = old_blending_mode_sub ;

  /* Attente d'une slection de mode de jeu. */
  if (wait_intro_input (&selected_game_type) == false)
    goto dirty_restart_intro ;

  /* Disparition de l'cran en fading. */
  old_blending_mode = BLEND_CR ;
  old_blending_mode_sub = SUB_BLEND_CR ;
  BLEND_Y = 0 ;
  SUB_BLEND_Y = 0 ;
  BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  for (i = 0; i < 32; i++) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
    SUB_BLEND_Y = (u16) (i / 2) ;
  }

  BLEND_CR = old_blending_mode ;
  SUB_BLEND_CR = old_blending_mode_sub ;

  videoSetMode (0) ;
  videoSetModeSub (0) ;

  return (selected_game_type) ;
}
