#include <nds.h>
#include <stdlib.h>
#include <string.h>
#include "basics.h"
#include "global.h"
#include "time.h"
#include "walls_gfx.h"
#include "filling.h"
#include "game_top_gfx.h"
#include "printing.h"
#include "sprite_laughing_ball_gfx.h"
#include "laugh.h"
#include "pause.h"
#include "goodies.h"
#include <nds/arm9/sound.h>
#include "wav_bump.h"
#include "wav_bump_radioac.h"
#include "wav_saber.h"
#include "wav_micro_active.h"
#include "inter_cpus.h"




/* Sous-routine de test de collision avec les bords et les murs.*/
/* N'affecte ni la position la vitesse des balles. */
/* Retourne ce que l'on a ventuellement touch (mur, bord ou libre). */
/* Elle gre aussi la destruction des murs (pas bords) en cas de mode */
/* "perforant" activ. */
static u8 collide (struct point *from, struct point *delta,
		   struct game *game)
{
  int top_left_x = (from->x + delta->x) ;
  int top_left_y = (from->y + delta->y) ;
  int bottom_right_x = (top_left_x + BALL_WIDTH) / 8 ;
  int bottom_right_y = (top_left_y + BALL_HEIGHT) / 8 ;
  top_left_x = top_left_x / 8 ;
  top_left_y = top_left_y / 8 ;

  if ((top_left_x < 0) || (top_left_y < 0) ||
      (bottom_right_x >= BOARD_WIDTH) || (bottom_right_y >= BOARD_HEIGHT))
    return (CELL_BORDER) ;

   int ul = game->board[top_left_x][top_left_y] ;
   int ur = game->board[top_left_x][bottom_right_y] ;
   int bl = game->board[top_left_x][bottom_right_y] ;
   int br = game->board[bottom_right_x][bottom_right_y] ;

   /* Un mur en construction. */
   if ((ul == CELL_WALL) || (ur == CELL_WALL) ||
       (bl == CELL_WALL) || (br == CELL_WALL))
     return (CELL_WALL) ;

   /* Espace libre. */
   if ((ul == CELL_FREE) && (ur == CELL_FREE) &&
       (bl == CELL_FREE) && (br == CELL_FREE))
     return (CELL_FREE) ;

   /* Ne reste qu'un bord "dur". */
   /* En cas de mode "perforant, on gre la destruction des blocs. */
   if (game->status.status &
       (GOODIES_MASK_TO_STATUS_MASK (Goodies_perfor_balls))) {
     if (ul == CELL_BORDER) {
       rem_wall_block (game->board, top_left_x, top_left_y) ;
       game->nb_filled_cells-- ;
     }
     if (ur == CELL_BORDER) {
       rem_wall_block (game->board, top_left_x, bottom_right_y) ;
       game->nb_filled_cells-- ;
     }
     if (bl == CELL_BORDER) {
       rem_wall_block (game->board, top_left_x, bottom_right_y) ;
       game->nb_filled_cells-- ;
     }
     if (br == CELL_BORDER) {
       rem_wall_block (game->board, bottom_right_x, bottom_right_y) ;
       game->nb_filled_cells-- ;
     }
   }
   return (CELL_BORDER) ;
}




/* Vrifie s'il y a besoin de dfaire une ligne en cours */
/* de formation et le cas chant la dfait. */
static void check_revert (struct game *game, bool lethal)
{
  struct liner *liner = &game->liner ;
  u16 i ;

  switch (liner->orient) {
  case Line_none: /* Rien n'a faire. Ne devrait pas se produire. */ break ;

  case Line_h:
    switch (liner->extrems) {
    case NONE_MASK:
      /* Il faut dtruire toute la ligne horizontale. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_neg_coord;
	   i <= liner->current_moving_pos_coord;
	   i++)
	rem_wall_block (game->board, i, liner->origin.y) ;
      break ;          // Fin de case NONE_MASK
    case POS_MASK:
      /* Il faut dtruire de la gauche vers l'origine exclue. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_neg_coord; i < liner->origin.x ; i++)
	rem_wall_block (game->board, i, liner->origin.y) ;
      break ;          // Fin de case POS_MASK
    case NEG_MASK:
      /* Il faut dtruire de l'origine exclue vers la droite. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_pos_coord; i > liner->origin.x ; i--)
	rem_wall_block (game->board, i, liner->origin.y) ;
      break ;          // Fin de case NEG_MASK
    case BOTH_MASK:
      /* Rien n'a faire. On accepte. */
      break ;          // Fin de case BOTH_MASK
    }                  // Fin de switch (liner->extrems)
    break ;            // Fin de case Line_h

  case Line_v:
    switch (liner->extrems) {
    case NONE_MASK:
      /* Il faut dtruire toute la ligne verticale. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_neg_coord;
	   i <= liner->current_moving_pos_coord;
	   i++)
	rem_wall_block (game->board, liner->origin.x, i) ;
      break ;          // Fin de case NONE_MASK
    case POS_MASK:
      /* Il faut dtruire du haut vers l'origine exclue. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_neg_coord; i < liner->origin.y ; i++)
	rem_wall_block (game->board, liner->origin.x, i) ;
      break ;          // Fin de case POS_MASK
    case NEG_MASK:
      /* Il faut dtruire de l'origine exclue vers le bas. */
      if (lethal) {
	playGenericSound (g_wav_saber, WAV_SABER_LENGTH * sizeof (u32)) ;
	slide_sprite
	  (sprite_laughing_ball_palette, SPRITE_LAUGHING_BALL_PALETTE_SIZE,
	   sprite_laughing_ball_tiles, SPRITE_LAUGHING_BALL_NB_TILES) ;
	game->lives-- ;
      }
      else
	playGenericSound
	  (g_wav_micro_active, WAV_MICRO_ACTIVE_LENGTH * sizeof (u32)) ;
      game->player.input_handled = Work_done ;
      liner->orient = Line_none ;
      for (i = liner->current_moving_pos_coord; i > liner->origin.y ; i--)
	rem_wall_block (game->board, liner->origin.x, i) ;
      break ;          // Fin de case NEG_MASK
    case BOTH_MASK:
      /* Rien n'a faire. On accepte. */
      break ;          // Fin de case BOTH_MASK
    }                  // Fin de switch (liner->extrems)
    break ;            // Fin de case Line_v
  }
}




/* Routine  principale de test de collision avec les bords et les murs.*/
/* Affecte uniquement le vecteur vitesse des balles. */
/* Elle modifie sont comportement normal alatoirement en faisant croire */
/* qu'on a heurt un BORDER afin de simuler l'effet "Dizzy". */
static void sprite_board_collision_check_and_react (struct ball *ball,
						    struct game *game)
{
  u32 last_time_called = 0 ;
  bool reflect_x = false ;
  bool reflect_y = false ;
  u8 coll_result ;
  struct point delta ;

  /* Test de collision en mouvement horizontal sur une paroie vertivale. */
  delta.x = ball->direction.x ;
  delta.y = 0 ;
  coll_result = collide (&ball->position, &delta, game) ;
  if (coll_result != CELL_FREE) {
    reflect_x = true ;
    if (coll_result == CELL_WALL)
      check_revert (game, true) ;
  }

  /* Test de collision en mouvement vertical sur une paroie horizontale. */
  delta.x = 0 ;
  delta.y =  ball->direction.y ;
  coll_result = collide (&ball->position, &delta, game) ;
  if (coll_result != CELL_FREE) {
    reflect_y = true ;
    if (coll_result == CELL_WALL)
      check_revert (game, true) ;
  }

  /* Test de collision en diagonal. */
  delta.x = ball->direction.x ;
  delta.y = ball->direction.y ;
  if ((! reflect_x) && (! reflect_y)) {
    coll_result = collide (&ball->position, &delta, game) ;
    if (coll_result != CELL_FREE) {
      reflect_x = reflect_y = true ;
      if (coll_result == CELL_WALL)
	check_revert (game, true) ;
    }
  }

  /* Si aucune collision mais "Dizzy mode" actif, alors */
  /* on va alatoirement gnrer une collision avec 1   */
  /* chance sur 2 qu'elle ait un effet.                 */
  if ((game->status.status &
       (GOODIES_MASK_TO_STATUS_MASK (Goodies_dizzy_balls))) &&
      ((! reflect_x) && (! reflect_y))) {
    u8 rnd_v = (u8) ((rand () + DISP_Y) % 6) ;
    
    /* 1 ventuel changement de direction toutes les 2.5 secondes. */
    if (elapsed_semi_seconds > (last_time_called + 4)) {
      switch (rnd_v) {
      case 0:
	reflect_x = true ;        /* Inversion horizontale. */
	break ;
      case 1:
	reflect_y = true ; 	  /* Inversion verticale. */
	break ;
      case 2:
	/* Inversion horizontale ET verticale. */
	reflect_x = true ;
	reflect_y = true ;
	break ;
      default:
	/* Aucun changement. */
	break ;
      }  /* Fin de switch (rnd_v). */
    }    /* Fin de if (elapsed_seconds > last_time_called). */
  }      /* Fin de if ((! reflect_x) && (! reflect_y)). */

  if (reflect_x) ball->direction.x = -ball->direction.x ;
  if (reflect_y) ball->direction.y = -ball->direction.y ;

  /* Un petit coup de son. */
  if ((reflect_x) || (reflect_y)) {
    if (game->status.status &
	(GOODIES_MASK_TO_STATUS_MASK (Goodies_perfor_balls)))
      playGenericSound (g_wav_bump_radioac,
			WAV_BUMP_RADIOAC_LENGTH * sizeof (u32)) ;
    else playGenericSound (g_wav_bump, WAV_BUMP_LENGTH * sizeof (u32)) ;
}
  last_time_called = elapsed_semi_seconds ;
}





static u8 get_cell (u8 board[BOARD_WIDTH][BOARD_HEIGHT], int x, int y)
{
  if ((x < 0) || (y < 0) || (x >= BOARD_WIDTH) || (y >= BOARD_HEIGHT))
    return (CELL_WALL) ;

  return (board[x][y]) ;
}




/* Routine principale de mouvement des balles. Affecte uniquement la */
/* position des balles. */
static void step_animate (struct game *game)
{
  struct ball *balls = game->balls ;
  unsigned short i ;
  u8 nb_balls = game->nb_balls ;

  for (i = 0; i < nb_balls; i++) {
    balls[i].position.x = balls[i].position.x + balls[i].direction.x ;
    balls[i].position.y = balls[i].position.y + balls[i].direction.y ;
  }

  for (i = 0; i < nb_balls; i++)
    sprite_board_collision_check_and_react (&balls[i], game) ;
}




/* Routine de test de collision entre 2 cellules et les balles. Change par */
/* effet de bord la valeur des cellules testes si une balle se trouve  */
/* cette position. Ces cellules ne doivent donc pas tre des alias dans la */
/* zone de jeu sinon a la modifiera. */
static void cell_versus_balls (u8 nb_balls, struct ball *balls, int x_prev,
			       int y_prev, int x_next, int y_next,
			       u8 *prev_cell, u8 *next_cell)
{
  u16 i ;
  int ball_x_as_block, ball_y_as_block ;
  bool changed_prev = false ;
  bool changed_next = false ;

  for (i = 0; i < nb_balls; i++) {
    ball_x_as_block = balls[i].position.x / 8 ;
    ball_y_as_block =balls[i].position.y / 8 ;
    if ((ball_x_as_block == x_prev) && (ball_y_as_block == y_prev)) {
      *prev_cell = CELL_BALL ;
      changed_prev = true ;
    }
    if ((ball_x_as_block == x_next) && (ball_y_as_block == y_next)) {
      *next_cell = CELL_BALL ;
      changed_next = true ;
    }
    if (changed_prev && changed_next) break ;
  }
}





static void update_board (struct game *game)
{
  static short remaining_delay = 0 ;
  struct player *player = &game->player ;
  struct liner *liner = &game->liner ;
  extremities previous_extrems = liner->extrems ;
  int coord_next, coord_prev ;
  u8 next, prev ;
  u8 step ;

  if (remaining_delay <= 0) {
    step = 1 ;
    remaining_delay = global_line_speed_delay ;
  }
  else {
    step = 0 ;
    remaining_delay-- ;
  }

  switch (liner->orient) {
  case Line_none: /* Rien n'a faire. */ break ;

  case Line_h:
    /* Progression d'une ligne horizontale. */
    coord_next = liner->current_moving_pos_coord + step ;
    next = get_cell (game->board, coord_next, liner->origin.y) ;
    coord_prev = liner->current_moving_neg_coord - step ;
    prev = get_cell (game->board, coord_prev, liner->origin.y) ;
    /* Par effet de bord, on va changer si ncessaire les cellules prev */
    /* et next pour vrifier si elles ne tombent pas sur une balle.     */
    cell_versus_balls (game->nb_balls, game->balls, coord_prev,
		       liner->origin.y, coord_next, liner->origin.y, &prev,
		       &next) ;

    switch (next) {
    case CELL_FREE:
      /* Il faut encore complter la ligne  droite. */
      add_wall_block (game->board, coord_next, liner->origin.y) ;
      liner->current_moving_pos_coord = coord_next ;
      break ;
    case CELL_BORDER: case CELL_LIMIT:
      CLOSE_POS (liner->extrems) ;
      break ;
    case CELL_BALL: break ;
    }       // Fin de switch (next)

    switch (prev) {
    case CELL_FREE:
      /* Il faut encore complter la ligne  gauche. */
      add_wall_block (game->board, coord_prev, liner->origin.y) ;
      liner->current_moving_neg_coord = coord_prev ;
      break ;
      
    case CELL_BORDER: case CELL_LIMIT:
      CLOSE_NEG (liner->extrems) ;
      break ;

    case CELL_BALL:
      check_revert (game, true) ;
      break ;
    }       // Fin de switch (prev)

    /* On regarde si la ligne a ferm la rgion. */
    switch (liner->extrems) {
    case NONE_MASK: /* Rien  faire. */ break ;

    case POS_MASK:
      /* La ligne a atteint le bord positif. */
      if (previous_extrems ^ liner->extrems) {
	/* Le changement vient de s'effectuer. */
	/* Remplissage en "dur" de la demi-droite. */
	hfill_border (game, liner->origin.x,
		      liner->current_moving_pos_coord, liner->origin.y) ;
      }          // Fin de if (previous_extrems ^ liner->extrems)
      break ;    // Fin de case POS_MASK

    case NEG_MASK:
      /* La ligne a atteint le bord ngatif. */
      if (previous_extrems ^ liner->extrems) {
	/* Le changement vient de s'effectuer. */
	/* Remplissage en "dur" de la demi-droite. */
	hfill_border (game, liner->current_moving_neg_coord,
		      liner->origin.x, liner->origin.y) ;
      }          // Fin de if (previous_extrems ^ liner->extrems)
      break ;    // Fin de case NEG_MASK

    case BOTH_MASK:
      /* La ligne VIENT d'atteindre les 2 bords. On durcit la ou les  */
      /* parties de ligne qui viennent de se terminer. Ce dernier cas */
      /* se produit dans les extrmits prcdentes taient toutes    */
      /* les deux ouvertes.                                           */
      if ((previous_extrems & POS_MASK) || (previous_extrems == NONE_MASK)) {
	/* C'est la portion ngative qui vient de se terminer. */
	hfill_border (game, liner->current_moving_neg_coord,
		      liner->origin.x, liner->origin.y) ;
      }
      if ((previous_extrems & NEG_MASK) || (previous_extrems == NONE_MASK)) {
	/* C'est la portion positive qui vient de se terminer. */
	hfill_border (game, liner->origin.x,
		      liner->current_moving_pos_coord, liner->origin.y) ;
      }
      /* Remplissage de l'aire. */
      fill_board (game) ;

      /* Reset du status du joueur. */
      player->input_handled = Work_done ;
      /* Reset du status du liner. */
      liner->orient = Line_none ;
      break ;       // Fin de case BOTH_MASK
    }               // Fin de switch (liner->extrems)
    break ;         // Fin de case Line_h


  case Line_v:
    /* Progression d'une ligne verticale. */
    coord_next = liner->current_moving_pos_coord + step ;
    next = get_cell (game->board, liner->origin.x, coord_next) ;
    coord_prev = liner->current_moving_neg_coord - step ;
    prev = get_cell (game->board, liner->origin.x, coord_prev) ;

    /* Par effet de bord, on va changer si ncessaire les cellules prev */
    /* et next pour vrifier si elles ne tombent pas sur une balle.     */
    cell_versus_balls (game->nb_balls, game->balls, liner->origin.x,
		       coord_prev, liner->origin.x, coord_next, &prev, &next) ;

    switch (next) {
    case CELL_FREE:
      /* Il faut encore complter la ligne en bas. */
      add_wall_block (game->board, liner->origin.x, coord_next) ;
      liner->current_moving_pos_coord = coord_next ;
      break ;
    case CELL_BORDER: case CELL_LIMIT:
      CLOSE_POS (liner->extrems) ;
      break ;
    case CELL_BALL:
      break ;
    }             /* Fin de switch (next). */

    switch (prev) {
    case CELL_FREE:
      /* Il faut encore complter la ligne en haut */
      add_wall_block (game->board, liner->origin.x, coord_prev) ;
      liner->current_moving_neg_coord = coord_prev ;
      break ;
    case CELL_BORDER: case CELL_LIMIT:
      CLOSE_NEG (liner->extrems) ;
      break ;

    case CELL_BALL:
      check_revert (game, true) ;
      break ;
    }             /* Fin de switch (prev). */

    /* On regarde si la ligne a ferm la rgion. */
    switch (liner->extrems) {
    case NONE_MASK: /* Rien  faire. */ break ;

    case POS_MASK:
      /* La ligne a atteint le bord positif. */
      if (previous_extrems ^ liner->extrems) {
	/* Le changement vient de s'effectuer. */
	/* Remplissage en "dur" de la demi-droite. */
	vfill_border (game, liner->origin.x,
		      liner->origin.y, liner->current_moving_pos_coord) ;
      }          /* Fin de if (previous_extrems ^ liner->extrems). */
      break ;    /* Fin de case POS_MASK. */

    case NEG_MASK:
      /* La ligne a atteint le bord ngatif. */
      if (previous_extrems ^ liner->extrems) {
	/* Le changement vient de s'effectuer. */
	/* Remplissage en "dur" de la demi-droite. */
	vfill_border (game, liner->origin.x,
		      liner->current_moving_neg_coord, liner->origin.y) ;
      }          /* Fin de if (previous_extrems ^ liner->extrems). */
      break ;    /* Fin de case NEG_MASK. */

    case BOTH_MASK:
      /* La ligne VIENT d'atteindre les 2 bords. On durcit la ou les  */
      /* parties de ligne qui viennent de se terminer. Ce dernier cas */
      /* se produit dans les extrmits prcdentes taient toutes    */
      /* les deux ouvertes.                                           */
      if ((previous_extrems & POS_MASK) || (previous_extrems == NONE_MASK)) {
	/* C'est la portion ngative qui vient de se terminer. */
	vfill_border (game, liner->origin.x,
		      liner->current_moving_neg_coord, liner->origin.y) ;
      }
      if ((previous_extrems & NEG_MASK) || (previous_extrems == NONE_MASK)) {
	/* C'est la portion positive qui vient de se terminer. */
	vfill_border (game, liner->origin.x,
		      liner->origin.y, liner->current_moving_pos_coord) ;
      }
      /* Remplissage de l'aire. */
      fill_board (game) ;
      /* Reset du status du joueur. */
      player->input_handled = Work_done ;
      /* Reset du status du liner. */
      liner->orient = Line_none ;
      game->nb_lines_done++ ;
      break ;     /* Fin de case BOTH_MASK. */
    }             /* Fin de switch (liner->extrems). */
  }               /* Fin de switch (liner->orient). */
}



static void draw_balls (u8 nb_balls, struct ball *balls)
{
  u16 i ;
  struct sSpriteEntry *oam_copy ;

  for (i = 0; i < nb_balls; i++) {
    oam_copy =  balls[i].oam_copy_data ;
    oam_copy->attribute[1] =
      (oam_copy->attribute[1] & 0xFE00) |(balls[i].position.x & 0x01FF) ;
    oam_copy->attribute[0] =
      (oam_copy->attribute[0] & 0xFF00) | (balls[i].position.y & 0x00FF) ;
  }
  
}





/* Code utilisant les fonctions de gestion du touchpad de */
/* la libnds (rcupration de position et de pression).   */
static bool check_user_pen (struct player *player, u16 status)
{
  u16 x, y ;
  bool ret_val = false ;
  touchPosition stylus_xy ;

  /* Si le joueur a dj une action en cours, on ne ragit pas au toucher. */
  if (player->input_handled == Work_todo) return (false) ;

  /*  Surtout pas de scanKeys () sinon a pte */
  /* la mmorisation au niveau infrieur.      */
  stylus_xy = touchReadXY () ;

  if (keysDown () & KEY_TOUCH) {
    x = stylus_xy.px ;
    y = stylus_xy.py ;
    player->start_touch.x = x ;
    player->start_touch.y = y ;
    ret_val = false ;
  }


  if (keysHeld () & KEY_TOUCH) {
    x = stylus_xy.px ;
    y = stylus_xy.py ;
    if (status & (GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_xy))) {
      player->end_touch.x = y ;
      player->end_touch.y = x ;
    }
    else {
      player->end_touch.x = x ;
      player->end_touch.y = y ;
    }
    ret_val = false ;
  }

  if (keysUp () & KEY_TOUCH) {
    player->input_handled = Work_todo ;
    ret_val = true ;
  }

  return (ret_val) ;
}





static void setup_liner (struct game *game)
{
  struct player *player = &game->player ;
  struct liner *liner = &game->liner ;
  int dx = abs (player->end_touch.x - player->start_touch.x) ;
  int dy = abs (player->end_touch.y - player->start_touch.y) ;
  int x_as_block = (player->start_touch.x / 8) ;
  int y_as_block = (player->start_touch.y / 8) ;

  if ((dx == dy) ||
      (get_cell (game->board, x_as_block, y_as_block) != CELL_FREE)) {
    /* Pas de direction claire ou pas de place. On abandonne. */
    player->input_handled = Work_done ;
    liner->orient = Line_none ;
    return ;
  }

  liner->origin.x = x_as_block ;
  liner->origin.y = y_as_block ;
  if (dx > dy) {
    /* Direction horizontale. */
    liner->orient = Line_h ;
    liner->current_moving_pos_coord =
      liner->current_moving_neg_coord = x_as_block ;
  }
  else {
    /* Il ne reste plus par dfaut que la direction verticale. */
    liner->orient = Line_v ;
    liner->current_moving_pos_coord =
      liner->current_moving_neg_coord = y_as_block ;
  }

  /* Maintenant, on trace le premier bloc de mur. */
  add_wall_block (game->board, x_as_block, y_as_block) ;
  OPEN_BOTH (liner->extrems) ;
}





enum end_code check_end_of_game (struct game *game)
{
  if (game->nb_filled_cells >=
      ((TO_FILL_PERCENT * (BOARD_WIDTH * BOARD_HEIGHT - 2)) / 100))
    return (Game_won) ;

  if (game->lives == 0) return (Game_lost) ;

  /* Test de la limite de temps. */
  if (game->kill_time <= elapsed_semi_seconds) {
    /* On prend la sanction. */
    switch (game->game_type) {
    case Game_standard:
      return (Game_restart) ;
      break ;

    case Game_arcade:
      launch_malus (game) ;
      game->kill_time = elapsed_semi_seconds + KILL_DELAY ;
      break ;
    }
  }
  return (Game_on) ;
}





static void check_end_of_goodies (struct game *game)
{
  u16 i ;
  u16 prev_status = game->status.status ;

  for (i = 0; i < NB_TIME_CONSTRAINTED_GOODIES; i++) {
    if (game->status.ends[i] < game->nb_lines_done)
      game->status.status &= ~(GOODIES_MASK_TO_STATUS_MASK (i)) ;
    }

  /* Cas particulier pour l'inversion d'cran. */
  if ((prev_status & (GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_screens))) &&
      (! (game->status.status &
	  (GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_screens))))) {
    /* Le goodies d'inversion d'cran vient de se terminer.  */
    lcdSwap () ;
  }

  /* Cas particulier pour le ralentissement des lignes. */
  if ((prev_status & (GOODIES_MASK_TO_STATUS_MASK (Goodies_slow_lines))) &&
      (! (game->status.status &
	  (GOODIES_MASK_TO_STATUS_MASK (Goodies_slow_lines))))) {
    /* Le goodies de ralentissement des lignes vient de se terminer.  */
    global_line_speed_delay = 0 ;
  }

  /* Cas particulier pour l'acclration des balles. */
  if ((prev_status & (GOODIES_MASK_TO_STATUS_MASK (Goodies_speed_balls))) &&
      (! (game->status.status &
	  (GOODIES_MASK_TO_STATUS_MASK (Goodies_speed_balls))))) {
    /* Le goodies d'acclration des balles vient de se terminer.  */
    for (i = 0; i < game->nb_balls; i++) {
      game->balls[i].direction.x = game->balls[i].initial_direction.x ;
      game->balls[i].direction.y = game->balls[i].initial_direction.y ;
    }
  }
}



enum end_code mainloop ()
{
  enum end_code game_status = Game_on ;

  /* Pour s'assurer que le buffer d'vnement touchpad est vide . */
  keysInit () ;
  for (;;) {
    /* Surtout 1 seul 'scanKeys' par itration. ne */
    /* pas en refaire un dans 'check_user_pen' !   */
    scanKeys () ;
    if (keysDown () & (KEY_START)) {
      if (pause_game (&global_game) == false) return (Game_lost) ;
    }

    /* Vrification de l'tat d'altration du jeu. */
    check_end_of_goodies (&global_game) ;

    /* Y a't-il eu du son d'enregistr ? */
    if (ipc_extension->sound_acquired == true) {
      u16 old_blending_mode_sub = SUB_BLEND_CR ;
      /* On fait un flash  l'cran. */
      SUB_BLEND_Y = 15 ;
      SUB_BLEND_CR = BLEND_FADE_WHITE | BLEND_SRC_BG0 ;
      /* On dtruit la ligne en cours, sans faire perdre de vie. */
      check_revert (&global_game, false) ;
      swiWaitForVBlank () ;
      SUB_BLEND_CR = old_blending_mode_sub ;
      ipc_extension->sound_acquired = false ; /* On le rinit. */
    }

    if (check_user_pen (&global_game.player, global_game.status.status)) {
      /* Il faut traiter la raction de l'utilisateur. */
      setup_liner (&global_game) ;
    }

    /* Mettre  jour le tableau et si ncessaire le joueur. Fait avancer  */
    /* la ligne ventuellement en cours de cration en grant sa vitesse. */
    update_board (&global_game) ;
    /* Vrification de la fin du jeu. */
    game_status = check_end_of_game (&global_game) ;
    if (game_status != Game_on) return (game_status) ;

    /* Animer un coup les balles. */
    step_animate (&global_game) ;

    /* Encore une vrification aprs le mouvement des balles. */
    game_status = check_end_of_game (&global_game) ;
    /* Vrification de la fin du jeu. */
    if (game_status != Game_on) return (game_status) ;
    /* Mise  jour des balles. */
    draw_balls (global_game.nb_balls, global_game.balls) ;

    /* Mise  jour des infos numriques  l'cran. */
    large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		     ((global_game.kill_time - elapsed_semi_seconds) / 2),
		     NUMERICS_X_OFFSET, TIMER_Y_OFFSET) ;
    large_print_int
      (((u16*) SCREEN_BASE_BLOCK (1)),
       (u32) global_game.score, NUMERICS_X_OFFSET, SCORE_Y_OFFSET) ;
    large_print_int
      (((u16*) SCREEN_BASE_BLOCK (1)),
       (u32) global_game.lives, NUMERICS_X_OFFSET, LIVES_Y_OFFSET) ;
    swiWaitForVBlank () ;
    /* Mise  jour graphique des sprites reprsentant les balles. */
    memcpy
      ((void*) OAM_SUB,
       (void*) global_sprites_copy, 128 * sizeof (struct sSpriteEntry)) ;
  }
}
