#include <nds.h>
#include <nds/arm9/sound.h>
#include "basics.h"
#include "time.h"
#include "gfx_load.h"
#include "init_game.h"
#include "pause_bottom_gfx.h"
#include "wav_screen_touch.h"
#include "game_top_gfx.h"
#include "printing.h"


#define PEN_HIT (pen_curr & ~pen_prev)

bool wait_for_continue ()
{
  u16 x, y ;
  touchPosition stylus_xy ;
  u16 pen_prev = false ;
  u16 pen_curr = false ;

  keysInit () ;
  for (;;) {
    scanKeys () ;
    stylus_xy = touchReadXY () ;
    pen_prev = pen_curr ;
    pen_curr = (keysDown () & KEY_TOUCH) ;

    /* Les touches font reprendre par dfaut. */
    if (keysDown () & (KEY_START | KEY_A | KEY_B)) return (true) ;

    if (PEN_HIT) {
      x = stylus_xy.px ;
      y = stylus_xy.py ;

      if ((x >= (BUTTONS_X * 8)) && (x <= ((BUTTONS_X + BUTTONS_WIDTH) * 8))) {
	/* On est positionn sur une abscisse de bouton. */
	if ((y >= BUTTON_CONTINUE_Y * 8) &&
	    (y < ((BUTTON_CONTINUE_Y + BUTTONS_HEIGHT) * 8))) {
	  playGenericSound
	    (g_wav_screen_touch, WAV_SCREEN_TOUCH_LENGTH * sizeof (u32)) ;
	  return (true) ;                    /* Continue. */
	}

	if ((y >= BUTTON_ABANDON_Y * 8) &&
	    (y < ((BUTTON_ABANDON_Y + BUTTONS_HEIGHT) * 8))) {
	  playGenericSound
	    (g_wav_screen_touch, WAV_SCREEN_TOUCH_LENGTH * sizeof (u32)) ;
	  return (false) ;                      /* Abandon. */
	}
      }   /* Fin de if ((x >= (BUTTONS_X * 8)) && (x <= ((BUTTOS_X + BUT ... */
    }
  }
}





bool pause_game (struct game *game)
{
  short i ;
  bool continue_game ;
  u16 old_blending_mode = BLEND_CR ;
  u16 old_blending_mode_sub = SUB_BLEND_CR ;
  bool was_flipped =
    game->status.status & (GOODIES_MASK_TO_STATUS_MASK (Goodies_flip_screens));

  /* On arrte le timer. */
  stop_timer2_no_count_seconds () ;

  videoSetMode (0) ;
  if (!was_flipped) lcdSwap () ;                /* On inverse les crans. */

  /* On grise un peu l'cran suprieur (de jeu). */
  SUB_BLEND_Y = 7 ;
  SUB_BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;

  /* cran infrieur. */
  load_palette_tile_map_256
    ((u16*) CHAR_BASE_BLOCK (1),     // TILES (16ko alignes) (adresse)
     (u16*) SCREEN_BASE_BLOCK (0),   // MAP (2ko aligne) (adresse)
     PAUSE_BOTTOM_PALETTE_SIZE,
     pause_bottom_palette,
     NB_PAUSE_BOTTOM_TILES,
     pause_bottom_tiles,
     PAUSE_BOTTOM_MAP_WIDTH,
     PAUSE_BOTTOM_MAP_HEIGHT,
     pause_bottom_map,
     Video_main) ;

  BG0_CR =
      BG_MAP_BASE (0)        // Map (pas adresse).
    | BG_TILE_BASE (1)       // Tiles (pas adresse).
    | BG_PRIORITY_3          // Priorit la plus faible.
    | BG_COLOR_256           // 256 couleurs.
    | TEXTBG_SIZE_256x256 ;  // 256 x 256 pixels

  /* Apparition de l'cran infrieur en fading. */
  BLEND_Y = 15 ;
  BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  videoSetMode (MODE_0_2D | DISPLAY_BG0_ACTIVE) ;
  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
  }

  continue_game = wait_for_continue () ;

  /* Disparition de l'cran infrieur en fading. */
  BLEND_Y = 0 ;
  BLEND_CR = BLEND_FADE_BLACK | BLEND_SRC_BG0 ;
  for (i = 0; i < 32; i++) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
  }

  videoSetMode (0) ;
  /* On recharge les graphismes de la patie "Scores" de l'cran de jeu. */
  init_playground_score_area () ;
  /* On remet les crans en place. */
  if (!was_flipped) lcdSwap () ;
  /* R-apparition de l'cran de score en fading. */
  BLEND_Y = 15 ;
  for (i = 31; i > 0; i--) {
    swiWaitForVBlank () ;
    BLEND_Y = (u16) (i / 2) ;
  }

  /* On remet les crans  leur luminosits normales. */
  BLEND_CR = old_blending_mode ;
  SUB_BLEND_CR = old_blending_mode_sub ;

  /* Remise  jour des infos numriques  l'cran. */
  large_print_int (((u16*) SCREEN_BASE_BLOCK (1)),
		   ((game->kill_time - elapsed_semi_seconds) / 2),
		   NUMERICS_X_OFFSET, TIMER_Y_OFFSET) ;
  large_print_int
    (((u16*) SCREEN_BASE_BLOCK (1)),
     (u32) game->score, NUMERICS_X_OFFSET, SCORE_Y_OFFSET) ;
  large_print_int
    (((u16*) SCREEN_BASE_BLOCK (1)),
     (u32) game->lives, NUMERICS_X_OFFSET, LIVES_Y_OFFSET) ;
  large_print_int
    (((u16*) SCREEN_BASE_BLOCK (1)),
     game->level, NUMERICS_X_OFFSET, LEVEL_Y_OFFSET) ;

  /* Et on rtablit l'cran de scores. */
  videoSetMode (MODE_0_2D | DISPLAY_BG0_ACTIVE | DISPLAY_BG1_ACTIVE) ;

  /* On remet le timer. */
  start_timer2_no_count_seconds () ;
  /* On efface toute manipulation rmanente des touche. */
  keysInit () ;
  return (continue_game) ;
}
